type GlobalContext<U = any> = {
  redis?: any;
} & U;

export const useContext = <T = GlobalContext>(initContext?: GlobalContext): T => {
  const context: GlobalContext = (global as any).context;
  const _context = context || initContext;
  if (!context) {
    if (_context) {
      (global as any).context = _context;
    } else {
      global.context = {};
    }
  }
  return global.context;
};

export const useContextKey = <T>(key: string, init: () => T): T => {
  const _context = useContext({});
  if (key && typeof _context[key] !== 'undefined') {
    return _context[key];
  }
  if (key && init) {
    _context[key] = init();
    return _context[key] as any;
  }
  return _context as any;
};

export const useContextKeySync = async <T = any>(key: string, init: () => Promise<T>): Promise<T> => {
  const _context = useContext({});
  if (key && typeof _context[key] !== 'undefined') {
    return _context[key];
  }
  if (key && init) {
    _context[key] = await init();
    return _context[key] as any;
  }
  return _context as any;
};

export const setContextKey = <T>(key: string, value: T) => {
  const _context = useContext<GlobalContext<T>>({});
  _context[key] = value;
  return _context;
};
